"""Demonstrate how to work with the absense of values in the X DevAPI
for MySQL Connector/Python.

The user that is used to connect to NySQL is the pyuser user, and the
connection is assumed to be made to localhost. The password is
Py@pp4Demo.

In a real program, please ensure the password is not stored inside the
souce code. The same is recommended for the rest of the connection
parameters.

The connection options can be changed in the connect_arge dictionary.

The user must have the CREATE, INSERT, SELECT, and DROP privileges on
the my_collections schema. An example that will create and grant the
correct privieleges to the pyuser@localhost is:

mysql> CREATE USER pyuser@localhost IDENTIFIED BY 'Py@pp4Demo';
mysql> GRANT CREATE, INSERT, SELECT, DROP
             ON my_collections.* TO pyuser@localhost;

For simplicity the error handling is kept to a minimum with just one
example of handling warnings included. In a real program, please make
sure to have proper error handling for all database operations.

This program is delivered as an example and as is. It is not meant for
production use. Please use at your own risk.
"""

import mysqlx

connect_args = {
    'host': '127.0.0.1',
    'port': 33060,
    'user': 'pyuser',
    'password': 'Py@pp4Demo',
};

# Create the database connection
db = mysqlx.get_session(**connect_args)

# Create the my_collections schema
schema = db.create_schema('my_collections')

# Create the my_docs collection in the my_collections schema
my_docs = schema.create_collection('my_docs')

# Define three documents to insert
kate = {
    'First_name': 'Kate',
    'Surname': 'Lee',
    'Birthday': '1982-08-09',
}

john = {
    'First_name': 'John',
    'Surname': 'Doe',
    'Birthday': None,
}


jane = {
    'First_name': 'Jane',
    'Surname': 'Doe'
}

# Insert the document for Adam
db.start_transaction()
result = my_docs.add(kate).add(john).add(jane).execute()
db.commit()

# Find documents by binding None
stmt_find = my_docs.find('Birthday = :birthday')
stmt_find.fields('First_name', 'Surname', 'Birthday')
stmt_find.bind('birthday', None)
result = stmt_find.execute()
print('People with Birthday bound to None:')
print('-' * 35)
for person in result.fetch_all():
    print("{First_name} {Surname} ({Birthday})".format(**person))

# Find documents where "Birthday IS NULL"
stmt_find = my_docs.find('Birthday IS NULL')
stmt_find.fields('First_name', 'Surname', 'Birthday')
result = stmt_find.execute()
print('')
print('People with Birthday IS NULL:')
print('-' * 29)
for person in result.fetch_all():
    print("{First_name} {Surname} ({Birthday})".format(**person))

# Find documents where Birthday is equal to a JSON document
# just containing a null value
stmt_find = my_docs.find("Birthday = CAST('null' AS JSON)")
stmt_find.fields('First_name', 'Surname', 'Birthday')
result = stmt_find.execute()
print('')
print("People with Birthday = CAST('null' AS JSON):")
print('-' * 44)
for person in result.fetch_all():
    print("{First_name} {Surname} ({Birthday})".format(**person))

# Close the database connection.
db.close()
